'use strict';
var path = require('path');
var util = require('util');
var yeoman = require('yeoman-generator');

var Generator = module.exports = function Generator() {
  yeoman.generators.Base.apply(this, arguments);
  this.argument('appname', { type: String, required: false });
  this.appname = this.appname || path.basename(process.cwd());

  var args = ['Main'];

  if (typeof this.env.options.appPath === 'undefined') {
    try {
      this.env.options.appPath = require(path.join(process.cwd(), 'component.json')).appPath;
    } catch (e) {}
    this.env.options.appPath = this.env.options.appPath || 'app';
  }
  this.appPath = this.env.options.appPath;

  if (typeof this.env.options.coffee === 'undefined') {
    this.option('coffee');

    // attempt to detect if user is using CS or not
    // if cml arg provided, use that; else look for the existence of cs
    if (!this.options.coffee &&
      this.expandFiles(process.cwd() + '/' + this.appPath + '/scripts/**/*.coffee', {}).length > 0) {
      this.options.coffee = true;
    }

    this.env.options.coffee = this.options.coffee;
  }

  this.hookFor('can:common', {
    args: args
  });

  this.hookFor('can:main', {
    args: args
  });

  this.hookFor('can:controller', {
    args: args
  });

  this.hookFor('can:model', {
    args: args
  });

  this.hookFor('can:view', {
    args: args
  });

  // Was part of the Angular generator setup and was used with mocha
  // TODO use karma with qunit
  this.hookFor('karma:app', {
    args: [false] // run karma hook in non-interactive mode
  });

  this.on('end', function () {
    console.log('\nI\'m all done. Just run ' + 'npm install && bower install --dev'.bold.yellow + ' to install the required dependencies.');
  });
};

util.inherits(Generator, yeoman.generators.NamedBase);

Generator.prototype.askFor = function askFor() {
  var cb = this.async();

  var prompts = [{
    name: 'bootstrap',
    message: 'Would you like to include Twitter Bootstrap?',
    default: 'Y/n',
    warning: 'Yes: All Twitter Bootstrap files will be placed into the styles directory.'
  }, {
    name: 'compassBootstrap',
    message: 'If so, would you like to use Twitter Bootstrap for Compass (as opposed to vanilla CSS)?',
    default: 'Y/n',
    warning: 'Yes: All Twitter Bootstrap files will be placed into the styles directory.'
  }];

  this.prompt(prompts, function(err, props) {
    if (err) {
      return this.emit('error', err);
    }

    this.bootstrap = (/y/i).test(props.bootstrap);
    this.compassBootstrap = (/y/i).test(props.compassBootstrap);
    cb();
  }.bind(this));
};

// Bunch of Angular add-ons
Generator.prototype.askForModules = function askForModules () {
  var cb = this.async();

  var prompts = [{
    name: 'qunit',
    message: 'Would you like to include JQuery\'s Qunit for unit testing?',
    default: 'Y/n',
    warning: 'Yes: Qunit added to component.json'
  }, {
    name: 'monsterTruck',
    message: 'Would you like to include Monster Trucks?',
    default: 'Y/n',
    warning: 'Yes: Monster trucks have been added to the project'
  }];

  this.prompt(prompts, function(err, props) {
    if (err) {
      return this.emit('error', err);
    }

    this.qunit = (/y/i).test(props.qunit);
    this.monsterTruck = (/y/i).test(props.monsterTruck);

    cb();
  }.bind(this));
};

// Duplicated from the SASS generator, waiting a solution for #138
Generator.prototype.bootstrapFiles = function bootstrapFiles() {
  var appPath = this.appPath;
  if (this.compassBootstrap) {
    var cb = this.async();

    this.write(path.join(appPath, 'styles/main.scss'), '@import "compass_twitter_bootstrap";');
    this.remote('vwall', 'compass-twitter-bootstrap', 'v2.2.2.2', function (err, remote) {
      if (err) {
        return cb(err);
      }
      remote.directory('stylesheets', path.join(appPath, 'styles') );
      cb();
    });
  } else if (this.bootstrap) {
    this.log.writeln('Writing compiled Bootstrap');
    this.copy( 'bootstrap.css', path.join(appPath, 'styles/bootstrap.css') );
  }

  if (this.bootstrap || this.compassBootstrap) {
    this.directory( 'images', 'app/images' );
  }
};

Generator.prototype.createIndexHtml = function createIndexHtml() {
  this.template('../../templates/common/index.html', path.join(this.appPath, 'index.html') );
};

Generator.prototype.packageFiles = function () {
  this.template( '../../templates/common/component.json', 'component.json' );
  this.template( '../../templates/common/package.json', 'package.json' );
  this.template( '../../templates/common/Gruntfile.js', 'Gruntfile.js' );
};
